--|----------------------------------------------------------------
--| Part of SmallEiffel -- Read DISCLAIMER file -- Copyright (C) 
--| Dominique COLNET and Suzanne COLLIN -- colnet@loria.fr
--|----------------------------------------------------------------

indexing

   description: "Characters, with comparison operations and an ASCII code.%
                %Note : corresponding C type is char";

	status: "See above notice";
	date: "$Date: 96/06/28 11:15:10 $";
	revision: "$Revision: 1.1 $";


expanded class CHARACTER

inherit 
   CHARACTER_REF
      redefine infix "<", infix "<=", infix ">", infix ">=", compare, 
         code, to_lower, to_upper, print_on
      end; 
   
feature {ANY}
   
   code: INTEGER is 
      -- ASCII code of Current.
      external "CSE"
      end; 
   
   infix "<"(other: CHARACTER): BOOLEAN is 
      -- Comparison using `code'. 
      external "CSE"
      end; 
   
   infix "<="(other: CHARACTER): BOOLEAN is 
      -- Comparison using `code'. 
      external "CSE"
      end; 
   
   infix ">"(other: CHARACTER): BOOLEAN is 
      -- Comparison using `code'. 
      external "CSE"
      end; 
   
   infix ">="(other: CHARACTER): BOOLEAN is 
      -- Comparison using `code'. 
      external "CSE"
      end; 
   
   compare(other: CHARACTER): INTEGER is 
      -- Compare Current with `other'.
      -- '<' <==> Result < 0
      -- '>' <==> Result > 0
      -- Otherwise Result = 0
      do  
         Result := code - other.code;
      end; 
   
   value: INTEGER is 
      -- Gives the value of a digit.
      require 
         is_digit; 
      do  
         Result := code - 48;
      ensure 
         0 <= Result and Result <= 9; 
         definition: Result = code - 48; 
      end; 
   
   same_as(other: CHARACTER): BOOLEAN is 
      -- No difference upper/lower.
      do  
         Result := to_lower = other.to_lower;
      ensure 
         Result implies to_lower = other or to_upper = other; 
      end; 
   
   to_upper: CHARACTER is 
      -- Conversion to the corresponding upper case.
      do  
         if code < 97 then 
            Result := Current;
         elseif code > 122 then 
            Result := Current;
         else 
            Result := (code - 32).to_character;
         end; 
      end; 
   
   to_lower: CHARACTER is 
      -- Conversion to the corresponding lower case.
      do  
         if code < 65 then 
            Result := Current;
         elseif code > 90 then 
            Result := Current;
         else 
            Result := (code + 32).to_character;
         end; 
      end; 
   
   is_lower: BOOLEAN is 
      -- Is `item' lowercase?
      do  
         inspect 
            Current
         when 'a'..'z' then 
               Result := true;
         else 
         end; 
      end; 
   
   is_upper: BOOLEAN is 
      -- Is `item' uppercase?
      do  
         inspect 
            Current
         when 'A'..'Z' then 
               Result := true;
         else 
         end; 
      end; 
   
   is_alpha, is_letter: BOOLEAN is 
      do  
         inspect 
            Current
         when 'A'..'Z','a'..'z' then 
               Result := true;
         else 
         end; 
      ensure 
         Result = ('A' <= Current and Current <= 'Z') or 'a' <= Current and Current <= 'z'; 
      end; 
   
   is_digit: BOOLEAN is 
      -- Belongs to '0'..'9'.
      do  
         inspect 
            Current
         when '0'..'9' then 
               Result := true;
         else 
         end; 
      ensure 
         Result implies '0' <= Current and Current <= '9'; 
      end; 
   
   is_separator: BOOLEAN is 
      -- True when character is a separator.
      do  
         inspect 
            Current
         when ' ','%T','%N','%R','%U' then 
               Result := true;
         else 
         end; 
      end; 
   
   is_letter_or_digit: BOOLEAN is 
      -- Is character a letter or digit?
      do  
         Result := is_letter or is_digit;
      ensure 
         valid: Result = (is_letter or is_digit); 
      end; 
   
   is_hex_digit: BOOLEAN is 
      -- Is character one of 0123456789abcdefABCDEF ?
      do  
         Result := is_digit or Current >= 'a' and Current <= 'f' or Current >= 'A' and Current <= 'F';
      ensure 
         valid: Result = (is_digit or Current >= 'a' and Current <= 'f' or Current >= 'A' and Current <= 'F'); 
      end; 
   
   is_horizontal_space: BOOLEAN is 
      -- Is character horizontal space?
      do  
         Result := Current = ' ' or Current = '%T';
      ensure 
         valid: Result = (Current = ' ' or Current = '%T'); 
      end; 
   
   is_vertical_space: BOOLEAN is 
      -- Is Character vertical space?
      do  
         Result := Current = '%N' or Current = '%F' or Current = '%R' or Current = '%/11/';
      ensure 
         valid: Result = (Current = '%N' or Current = '%F' or Current = '%R' or Current = '%/11/'); 
      end; 
   
   is_space: BOOLEAN is 
      -- Is character horizontal or vertical space?
      do  
         Result := is_horizontal_space or is_vertical_space;
      ensure 
         valid: Result = (is_horizontal_space or is_vertical_space); 
      end; 
   
   is_symbol: BOOLEAN is 
      -- Is character printable, but not letter or digit?
      do  
         Result := is_printable and not is_letter_or_digit;
      ensure 
         valid: Result = (is_printable and not is_letter_or_digit); 
      end; 
   
   is_punctuation: BOOLEAN is 
      -- Is character printable, but not a letter
      -- or a digit or a space?
      do  
         Result := is_printable and not is_letter_or_digit and Current /= ' ';
      ensure 
         valid: Result = (is_printable and not is_letter_or_digit and Current /= ' '); 
      end; 
   
   is_printable: BOOLEAN is 
      -- Is character a printing character?
      do  
         inspect 
            Current
         when '%/32/'..'%/126/','%/128/'..'%/255/' then 
               Result := true;
         else 
         end; 
      end; 
   
   is_control: BOOLEAN is 
      -- Is character a control character or DEL?
      do  
         Result := Current >= '%U' and Current <= '%/31/' or Current = '%/127/';
      ensure 
         valid: Result = (is_ascii and not is_printable); 
      end; 
   
   is_ascii: BOOLEAN is 
      -- Is character a 8-bit ASCII character?
      do  
         inspect 
            Current
         when '%U'..'%/255/' then 
               Result := true;
         else 
         end; 
      end; 
   
   is_visible: BOOLEAN is 
      -- Is character a printable character that isn't a space?
      do  
         Result := is_printable and Current /= ' ';
      ensure 
         valid: Result = (is_printable and Current /= ' '); 
      end; 
   
   print_on(file: STD_FILE_WRITE) is 
      do  
         file.put_character('%'');
         file.put_character(Current);
         file.put_character('%'');
      end; 

end -- class CHARACTER
